use serde::{Deserialize, Serialize};
use std::collections::HashMap;
use std::fs;
use std::fs::File;
use std::io::Write;
use tera::{to_value, try_get_value, Result, Tera, Value};

#[derive(Deserialize, Serialize)]
struct ReadmeConfig {
    header: Image,
    project: Project,
}

#[derive(Deserialize, Serialize)]
struct Project {
    name: String,
    summary: String,
    description: String,
    language: String,
    licence: String,
    lifecycle: String,
    badges: Vec<Image>,
}

#[derive(Deserialize, Serialize)]
struct Image {
    alt: String,
    href: String,
    src: String,
}

#[derive(Deserialize, Serialize)]
struct Language {
    color: String,
}

#[derive(Deserialize, Serialize)]
struct Licence {
    name: String,
    class: String,
    grant: String,
}

fn licence_badge(name: &Value, _: &HashMap<String, Value>) -> Result<Value> {
    let mut s = try_get_value!("licence_badge", "value", String, name);
    s = s.to_lowercase().replace("-", "--").replace(" ", "%20");
    Ok(to_value(s.as_str())?)
}

fn licence_colour(name: &Value, _: &HashMap<String, Value>) -> Result<Value> {
    let s = try_get_value!("licence_colour", "value", String, name);
    let result = match s.as_str() {
        "publicdomain" => "7cd958",
        "permissive" => "green",
        "copyleft" => "orange",
        "proprietary" => "red",
        _ => "grey",
    };
    Ok(to_value(result)?)
}

fn main() {
    let template = include_str!("template.md");
    let mut tera = Tera::default();
    tera.add_raw_template("template.md", template)
        .expect("failed to add template");
    tera.register_filter("licence_badge", licence_badge);
    tera.register_filter("licence_colour", licence_colour);

    let languages_toml = include_str!("languages.toml");
    let languages: HashMap<String, Language> = toml::from_str(languages_toml).unwrap();

    let licences_toml = include_str!("licences.toml");
    let licences: HashMap<String, Licence> = toml::from_str(licences_toml).unwrap();

    let toml_filename = "README.toml";
    let contents: String = fs::read_to_string(toml_filename).unwrap();
    let readme_config: ReadmeConfig =
        toml::from_str(&contents).expect("Failed to parse readme.toml");

    let mut context = tera::Context::new();
    context.insert("config", &readme_config);
    context.insert("language", &languages.get(&readme_config.project.language));
    context.insert("licence", &licences.get(&readme_config.project.licence));

    let rendered = tera.render("template.md", &context).unwrap();
    let mut file = File::create("README.md").expect("could not create README.md");
    file.write_all(rendered.as_bytes())
        .expect("failed to write to README.md");
}
